;---------------------------------------------------------------------------
; [C_CHARPUT_Clip] [AF, BC, DE, HL, IX] [178 bytes] [Hideaki Omuro & Kouri]
;---------------------------------------------------------------------------
; Draws a character in custom font to GRAPH_MEM with sprite clipping.
; Custom fonts must be 4 by 6 pixels per character (see 4x6font.inc).

; Preconditions:
; a = character, (b,c) = coords, where (0,0) is the top left corner of the
; screen and (95,63) is the bottom right corner.

; Postconditions:
; Character xor'd to GRAPH_MEM at (b,c) coords
; b = new x-coord.
; No other registers are modified.

C_CHARPUT_Clip:
 push af \ push bc \ push de \ push hl \ push ix

 ld e,a
 xor a
 ld d,a
 ld hl,Font			; Point to custom font
 add hl,de			; Add 3 times because each sprite is 3 bytes
 add hl,de
 add hl,de			; hl -> sprite

 ld e,6
 or c \ jp m,NoBottomClip
 ld a,$3F
 sub c \ ret c
 cp 5 \ jr nz,NoVertClip
 inc a
 ld e,a
 jr NoVertClip

NoBottomClip:
 cp -5 \ ret c
 add a,e
 ld c,a
 sub e
 neg
 ld e,a
 add hl,de
 ld e,c
 ld c,d

NoVertClip:
 push hl \ pop ix
 ld a,$77
 ld (OP_change_1),a
 ld (OP_change_2),a
 xor a
 or b \ jp m,NoRightClip
 cp 89 \ jr c,ClipDone
 cp 96 \ ret nc
 ld hl,OP_change_2
 jr ClipModify

NoRightClip:
 cp -7 \ ret c
 ld hl,OP_change_1

ClipModify:
 ld (hl),d

ClipDone:
 ld b,d
 ld h,b
 ld l,c
 add hl,hl
 add hl,bc
 add hl,hl
 add hl,hl
 ld c,a
 sra c \ sra c \ sra c
 add hl,bc
 ld bc,GRAPH_MEM
 add hl,bc
 ld b,e
 cpl
 and 00000111b
 ld e,a
 add a,e
 add a,e
 ld (C_CHARPUT_Shift+1),a
 ld b,3
 ld de,11

C_CHARPUT_Loop_1:
 push bc
 ld b,2
 ld a,(ix)
 and 11110000b
C_CHARPUT_Loop_2:
 ld c,d
C_CHARPUT_Shift:
 jr C_CHARPUT_Shift
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 xor (hl)
OP_change_1:
 ld (hl),a
 inc hl
 ld a,c
 xor (hl)
OP_change_2:
 ld (hl),a
 add hl,de
 ld a,(ix)
 add a,a \ add a,a \ add a,a \ add a,a
 djnz C_CHARPUT_Loop_2
 pop bc
 inc ix
 djnz C_CHARPUT_Loop_1
 pop ix \ pop hl \ pop de \ pop bc
 ld a,b \ add a,4 \ ld b,a
 pop af
 ret

;---------------------------------------------------------------------------

Font:
#include 4x6font.inc
