;---------------------------------------------------------------------------
; [ C_CHARPUT ] [ AF, BC, DE, HL, IX ] [ 116 bytes ]              [ Kouri ]
;---------------------------------------------------------------------------
; Draws a character in custom font to GRAPH_MEM (no clipping).
; Custom fonts must be 4 by 6 pixels per character (see 4x6font.inc).

; Preconditions:
; a = character, (b,c) = coords, where (0,0) is the top left corner of the
; screen and (95,63) is the bottom right corner.

; Postconditions:
; Character xor'd to GRAPH_MEM at (b,c) coords
; b = new x-coord.
; No other registers are modified.

C_CHARPUT:
 push af \ push bc \ push de \ push hl \ push ix

 ld d,0
 ld e,a
 ld hl,Font			; Point to custom font
 add hl,de			; Add 3 times because each sprite is 3 bytes
 add hl,de
 add hl,de
 push hl \ pop ix		; ix -> sprite for char
 ld a,b				; Save x-coord
 ld b,d
 ld h,b
 ld l,c
 add hl,hl			; 2y
 add hl,bc			; 3y
 add hl,hl			; 6y
 add hl,hl			; 12y
 ld c,a				; Restore x-coord
 srl c \ srl c \ srl c		; x/8
 add hl,bc
 ld bc,GRAPH_MEM
 add hl,bc
 cpl
 and 00000111b			; Get the remainder of x/8
 ld b,a
 add a,a
 add a,b			; Calculate number of jumps
 ld (C_CHARPUT_Shift+1),a	; Self-modifying code
 ld b,3
 ld de,11

C_CHARPUT_Loop:
 push bc
 ld b,2
 ld a,(ix)
 and 11110000b
C_CHARPUT_Loop2:
 ld c,d
C_CHARPUT_Shift:
 jr C_CHARPUT_Shift
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 rrca \ rr c
 xor (hl) \ ld (hl),a
 inc hl
 ld a,c
 xor (hl) \ ld (hl),a
 add hl,de
 ld a,(ix)
 add a,a \ add a,a \ add a,a \ add a,a
 djnz C_CHARPUT_Loop2
 pop bc
 inc ix
 djnz C_CHARPUT_Loop
 pop ix \ pop hl \ pop de \ pop bc
 ld a,b \ add a,4 \ ld b,a
 pop af
 ret

;---------------------------------------------------------------------------

Font:
#include 4x6font.inc
