;---------------------------------------------------------------------------
; [ D_ZC_STR ] [ AF, BC, DE, HL, IX ]                             [ Kouri ]
;---------------------------------------------------------------------------
; Draws a zero-terminated string in custom font to GRAPH_MEM.
; Custom fonts must be 4 by 6 pixels per character (see 4x6font.inc).
; D_ZC_STR stops drawing when it reaches the end of the string (signified
; by a zero) or if the x-coord goes offscreen. D_ZC_STR can be used with
; either C_CHARPUT or C_CHARPUT_Clip.

; Preconditions:
; hl -> string, (b,c) = coords, where (0,0) is the top left corner of the
; screen and (95,63) is the bottom right corner.

; Postconditions:
; String xor'd to GRAPH_MEM at (b,c) coords
; hl -> byte after last character written (may be zero), b = new x-coord
; No other registers are modified.

D_ZC_STR:
 push af \ push de \ push ix

D_ZC_STR_Loop:
 ld a,(hl)			; Get char into a
 inc hl				; Point to next char
 or a \ jr z,D_ZC_STR_Done	; If you want registers saved
 call C_CHARPUT			; Draw the character
; call C_CHARPUT_Clip		; Draw the character with clipping
 ld a,b
 cp 96 \ jr c,D_ZC_STR_Loop	; If x-coord > 95, we're done

D_ZC_STR_Done:
 pop ix \ pop de \ pop af
 ret

#include CCHARPUT.inc
;#include CCHARClp.inc
